--[[
************************************************************************

icon.lua

File date: 2019-12-04T19:33:33Z
File revision: 
Project revision: 293
Project version: r293


Author: Angelike

************************************************************************
]]--

local addonName = ... .. 'icon';

local beql = LibStub("AceAddon-3.0"):GetAddon("beql");
local beqlIcon = beql:NewModule(addonName);

local L = LibStub("AceLocale-3.0"):GetLocale("beql");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

local beqlLDB = LibStub("LibDataBroker-1.1"):NewDataObject(addonName, {
    type = "data source",
    text = "beql",
    icon = "Interface\\Icons\\INV_Letter_03"
})

local iconFrame = LibStub("LibDBIcon-1.0")

-- Fubar Options

local beqlfubar = {
    desc = L["Icon Config"],
    name = L["Icon Config"],
    type = 'group',
    args = {
        hideplugin = {
            desc = L["Hide the plugin from the panel or minimap, leaving the addon running."],
            get = function(info)
                return beql.db.profile.fubar.hide
            end,
            name = L["Hide plugin"],
            order = 1,
            set = function(info, newval)
                beql.db.profile.fubar.hide = newval;
                iconFrame:Refresh(addonName);
            end,
            type = 'toggle',
        },
        headline = {
            order = 2,
            type = 'header',
        },
        ShowTooltip = {
            desc = L["Shows the tooltip on mousover this plugin"],
            disabled = function(info)
                return beql.db.profile.fubar.hide
            end,
            get = function(info)
                return beql.db.profile.fubar.tooltip
            end,
            name = L["Show Icon Tooltip"],
            order = 6,
            set = function(info, newval)
                beql.db.profile.fubar.tooltip = newval
            end,
            type = 'toggle',
        },
        MinimapPosition = {
            desc = L["Minimap position"],
            disabled = function(info)
                return beql.db.profile.fubar.hide
            end,
            get = function(info)
                return beql.db.profile.fubar.minimapPos
            end,
            min = 1,
            max = 360,
            name = L["Minimap position"],
            order = 10,
            set = function(info, newval)
                beql.db.profile.fubar.minimapPos = newval;
                iconFrame:Refresh(addonName);
            end,
            step = 1,
            type = 'range',
        }
    }
}

function beqlIcon:OnInitialize()
    iconFrame:Register(addonName, beqlLDB, beql.db.profile.fubar);

    local mainOptions = beql:getMainOptionsTable();

    mainOptions.args.fubar = beqlfubar;
    beql.optionsFrames.fubar = AceConfigDialog:AddToBlizOptions(beql.baseName, L["Icon Config"], beql.baseName, "fubar");

    iconFrame:Hide(addonName)
end

function beqlLDB:OnClick(button) 
    if (IsShiftKeyDown()) then
        beql:OpenBlizzardOption("main")
    elseif (button == 'RightButton') then
        beql:OpenBlizzardOption("fubar")
    else
        ToggleFrame(QuestLogFrame);
    end
end
    
function beqlLDB:OnTooltipShow()
    if (not beql.db.profile.fubar.tooltip) then
        return;
    end

    iconFrame.tooltip:SetOwner(iconFrame:GetMinimapButton(addonName), "ANCHOR_BOTTOMLEFT", 30, 0)

    local questEntries = select(2, GetNumQuestLogEntries())
    local numfinished = 0 --select(2, beqlQ:GetNumQuests())

    -- title
    iconFrame.tooltip:AddLine(addonName)
    iconFrame.tooltip:AddLine(" ") -- spacer

    -- num quests
    iconFrame.tooltip:AddDoubleLine(L["|cffffffffQuests|r"], questEntries)

    -- num watched quests
    iconFrame.tooltip:AddDoubleLine(L["|cffff8000Tracked Quests|r"], GetNumQuestWatches())

    -- num completed quests
    iconFrame.tooltip:AddDoubleLine(L["|cff00d000Completed Quests|r"], numfinished)
    iconFrame.tooltip:AddLine(" ") -- spacer

    -- tooltip hint
    iconFrame.tooltip:AddLine(L["|cffeda55fClick|r to open Quest Log."] .. ' ' .. L["|cffeda55fShift+Click|r to open beql config."] .. ' ' .. L["|cffeda55fRightclick|r to open Icon config."], 0, 1, 0, true)

    iconFrame.tooltip:Show()
end

-- EOF --
