--[[
************************************************************************

tooltip.lua

File date: 2019-12-04T19:33:33Z
File revision: 
Project revision: 293
Project version: r293


Author: Angelike

************************************************************************
]]--

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")

local GameTooltip = GameTooltip

local isFading, onFadeEnd, stopFading, startFading;

local hooked

local GameTooltipQuests = {}

local function RGBPercToHex(r, g, b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

--[[
--------------------------------------------
-- Notes:
-- Init the Tooltips for beql
--
-- Returns:
-- * nothing
--------------------------------------------]]
function beql:Tooltip_InitTooltip()
	if (not hooked) then
		-- do some GameTooltip hooks
		GameTooltip:HookScript("OnShow", beql.Hooks_GameTooltip_OnShow)
		GameTooltip:HookScript("OnTooltipSetItem", beql.Hooks_GameTooltip_OnTooltipSetItem)
		GameTooltip:HookScript("OnTooltipSetUnit", beql.Hooks_GameTooltip_OnTooltipSetUnit)
		hooked = true
	end

    beql.frames.questWatchTooltip.IsFading = isFading;
    beql.frames.questWatchTooltip.FadeOutTooltip = startFading;
    beql.frames.questWatchTooltip.StopFading = stopFading;
end


--
-- Hooks
--

function beql.Hooks_GameTooltip_OnShow(this)
	if (not beqlQ or not beql:IsEnabled() or not beql.db.profile.Tooltip.mob) then
		-- not need
		return
	end

	-- not a mob, item or spell, may an object, check it for queststrings
	if (not GameTooltip:GetUnit() and not GameTooltip:GetItem() and not GameTooltip:GetSpell() and GameTooltip:NumLines() > 0 and GameTooltip:IsShown()) then
		if (beql:Tooltip_ScanTooltip()) then
			local maxwidth = 0
			for i=1, GameTooltip:NumLines() do
				maxwidth = max(_G["GameTooltipTextLeft" .. i]:GetWidth(), maxwidth)
			end

			-- adjust width
			if (GameTooltip:GetWidth() - maxwidth) < 20 then
				GameTooltip:SetWidth(maxwidth + 20.48)
			end
		end
	end
end

--[[
--------------------------------------------
-- Notes:
-- Adds a Quest on Questmob if not exists
--
-- Returns:
-- * nothing
--------------------------------------------]]
function beql:Hooks_GameTooltip_OnTooltipSetUnit()

	if (not beqlQ or not beql:IsEnabled() or not beql.db.profile.Tooltip.mob) then
		-- not need
		return;
	end

	beql:Tooltip_ScanTooltip();

	local uID;
	local queryString = GameTooltip:GetUnit();
	if (queryString) then
		if (beqlQ:IsQuestMob(queryString)) then
			uID = beqlQ.quest_mobs[queryString];
		end
	end

	-- questmob not found, may we must "collect" the mob as item? (i.e. "Snowfall Glade Pup")
	if (not uID and beqlQ:IsQuestItem(queryString)) then
		uID = beqlQ:IsQuestItem(queryString);
	end

	if (uID and type(uID) == 'number') then
		local found = false;
		for _, qid in pairs (GameTooltipQuests) do
			if (uID == qid) then
				-- quest already in tooltip
				found = true;
			end
		end
		if (not found) then

			-- not in tooltip, add it
			local qID, _ , qTitle, qLvl, qTag, qObj, qStat, qRec, qdaily, qZone = beqlQ:GetQuestByUid(uID);

			-- color
            local color;
            if (beql.db.profile.Tooltip.GameTooltip_TitelIsQuestcolor) then
                color = GetQuestDifficultyColor(qLvl);
            else
                color = CopyTable(beql.db.profile.Tooltip.Colors.GameTooltip_MobTitle);
            end

			if (beql.db.profile.QuestLog.showlevel) then
                qTag = " [" .. qLvl .. string.upper(beqlQ:GetShortTagForQuest(qID)).."] ";
			else
                qTag = " ";
			end

			GameTooltip:AddLine(qTag .. qTitle, color.r, color.g, color.b)
			for objname, objval in pairs(beqlQ.quest_objectives[uID]) do
				if (objname == queryString) then
					local objP, objN, _ = unpack(objval)
                    --					GameTooltip:AddLine("  - " .. objname .. ": " .. objP .. "/" .. objN, color.r, color.g, color.b)
					-- objective
					if (beql.db.profile.Tooltip.GameTooltip_ColorObjectives) then
						if (beql.db.profile.Tooltip.GameTooltip_FadeObjectiveColors) then
							local percent = beql:round((objP/objN)*100, 2)
							if (beql.db.profile.Tooltip.GameTooltip_ObjectiveUseStep) then
								if (percent < 50) then
									color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveStep, objP, objN/2)
								else
									color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveStep, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete, objP-(objN/2), objN)
								end
							else
								color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete, objP, objN)
							end
						else
							if (objP ~= objN) then
								color = beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete
							else
								color = beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete
							end
						end
					end
					GameTooltip:AddLine("  - " .. objname .. ": " .. objP .. "/" .. objN, color.r, color.g, color.b)
				end
			end

			-- perform a manual Tooltipupdate out of updatetime
			GameTooltip:GetScript("OnUpdate")(GameTooltip, 1)
		end
	end

end

--[[
--------------------------------------------
-- Notes:
-- Adds a Quest on Questitem if not exist
--
-- Returns:
-- * nothing
--------------------------------------------]]
function beql:Hooks_GameTooltip_OnTooltipSetItem()
	if (not beqlQ or not beql:IsEnabled() or not beql.db.profile.Tooltip.item) then
		-- not need
		return
	end

    --		beql:Tooltip_ScanTooltip() -- may in future
	GameTooltipQuests = {}

	local queryString = GameTooltip:GetItem()
	if (queryString) then
		if (beqlQ:IsQuestItem(queryString)) then
			local uID, objName, objP, objN, _ = beqlQ:IsQuestItem(queryString)
			local found = false
			if (GameTooltipQuests and #GameTooltipQuests > 0) then
				for _, qid in pairs (GameTooltipQuests) do
					if (uID == qid) then
						-- quest already in tooltip
						found = true
					end
				end
			end
			if (not found) then
				-- not in tooltip, add it
				local qID, _ , qTitle, qLvl, qTag = beqlQ:GetQuestByUid(uID)

				-- color
				local color
				if (beql.db.profile.Tooltip.GameTooltip_TitelIsQuestcolor) then
					color = GetQuestDifficultyColor(qLvl)
				else
					color = CopyTable(beql.db.profile.Tooltip.Colors.GameTooltip_ItemTitle)
				end

				if (beql.db.profile.QuestLog.showlevel) then
                    qTag = " [" .. qLvl .. string.upper(beqlQ:GetShortTagForQuest(qID)).."] "
				else
                    qTag = " "
				end


				
				GameTooltip:AddLine(qTag .. qTitle, color.r, color.g, color.b)
				-- objective
				if (beql.db.profile.Tooltip.GameTooltip_ColorObjectives) then
					if (beql.db.profile.Tooltip.GameTooltip_FadeObjectiveColors) then
						local percent = beql:round((objP/objN)*100, 2)
						if (beql.db.profile.Tooltip.GameTooltip_ObjectiveUseStep) then
							if (percent < 50) then
								color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveStep, objP, objN/2)
							else
								color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveStep, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete, objP-(objN/2), objN)
							end
						else
							color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete, objP, objN)
						end
					else
						if (objP ~= objN) then
							color = beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete
						else
							color = beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete
						end
					end
				end
				GameTooltip:AddLine("  - " .. objName .. ": " .. objP .. "/" .. objN, color.r, color.g, color.b)
			end
		end
	end
end

--
-- User Functions
--

--[[
--------------------------------------------
-- Notes:
-- Scans Tooltip for any Quests
--
-- Returns:
-- * boolean - true if found a quest
--------------------------------------------]]
function beql:Tooltip_ScanTooltip()
	GameTooltipQuests = {}
	local found, qstat = false, false
	local questline
	local line, hexcolor, linetext
	local qID, qTitle, qLvl, qTag, qComplete, _
	local color = {}
	local Color = {}
	for i= 1, GameTooltip:NumLines() do
		if (i > 1) then
			line = _G["GameTooltipTextLeft" .. i]
			Color = {}
			Color.r, Color.g, Color.b, Color.a = line:GetTextColor()
			hexcolor = RGBPercToHex(Color.r, Color.g, Color.b)
			if (hexcolor == "fed100") then
				-- may a Quest
				linetext = line:GetText()
				for key, quest in pairs(beqlQ.quests) do
					-- search quest by name
					if (quest.title == linetext) then
						-- Quest found!
						found = true
						qID = key;
						table.insert(GameTooltipQuests, qID)
						qTitle, qLvl, _, _, qComplete = select(3, beqlQ:GetQuestByUid(qID))

						if (qComplete == 2) then
							qstat = 1
							line:SetHeight(0.01)
							line:SetTextColor(0, 0, 0, 0)
							line:SetText(" ")
							line:SetPoint("TOPLEFT", _G["GameTooltipTextLeft" .. (i-1)], 0, -2)
						else
							qstat = 0
							local color
							if (beql.db.profile.Tooltip.GameTooltip_TitelIsQuestcolor) then
								color = GetQuestDifficultyColor(qLvl)
							else
								color = CopyTable(beql.db.profile.Tooltip.Colors.GameTooltip_MobTitle)
							end

							if (beql.db.profile.QuestLog.showlevel) then
                                qTag = " [" .. qLvl .. string.upper(beqlQ:GetShortTagForQuest(qID)).."] "
							else
                                qTag = " "
							end
							qTitle = qTag .. line:GetText()
							line:SetText(qTitle)-- .. " (QuestId: " .. tostring(key) .. ")")
							line:SetTextColor(color.r, color.g, color.b)
						end

					end
				end
			elseif (qstat and (hexcolor == 'fefefe') and strmatch(line:GetText(), '^ (\-) ')) then
				if (qstat == 1) then
					line:SetHeight(0.01)
					line:SetTextColor(0, 0, 0, 0)
					line:SetText(" ")
					line:SetPoint("TOPLEFT", _G["GameTooltipTextLeft" .. (i-1)], 0, -2)
				else
					if (beql.db.profile.Tooltip.GameTooltip_ColorObjectives) then
						local color
						local has, need = string.match(line:GetText(), " - \.-: (%d+)/(%d+)")
						if (has and need) then
							if (beql.db.profile.Tooltip.GameTooltip_FadeObjectiveColors) then
								local percent = beql:round((has/need)*100, 2)
								if (beql.db.profile.Tooltip.GameTooltip_ObjectiveUseStep) then
									if (percent < 50) then
										color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveStep, has, need/2)
									else
										color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveStep, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete, has-(need/2), need)
									end
								else
									color = beql:FadeColors(beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete, beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete, has, need)
								end
							else
								if (has ~= need) then
									color = beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete
								else
									color = beql.db.profile.Tooltip.Colors.GameTooltip_ObjectiveComplete
								end
							end
						else
							color = beql.db.profile.Tooltip.Colors.GameTooltip_NonObjective
						end
						line:SetTextColor(color.r, color.g, color.b)
					end
					-- quest objective
					line:SetText(" " .. line:GetText())
				end
			elseif ( line:GetText() ~= qTitle ) then
				qstat = nil;
			end
		end
		line, linetext, hexcolor = nil, nil, nil;
	end
	return found
end

function isFading(self)
    return self.fadeInfo ~= nil;
end

function onFadeEnd()
    local tooltip = beql.frames.questWatchTooltip;

    tooltip:ClearLines();
    tooltip:Hide();
    tooltip:SetAlpha(1);
    tooltip.fadeInfo = nil;
end

function startFading(self, time)
    ResetCursor();

    local fadeInfo = {
        fadeHoldTime = 1,
        finishedFunc = onFadeEnd,
        mode = "OUT",
        timeToFade = time or 0.25
    };

    UIFrameFade(self, fadeInfo);
end

function stopFading(self)
    UIFrameFadeRemoveFrame(self);
    onFadeEnd();
end

--[[
--------------------------------------------
-- Notes:
-- Shows Tooltip on Questtracker
--
-- Returns:
-- * nothing
--------------------------------------------]]

function beql:QuestTracker_OnUpdate(elapsed)
    local tooltip = beql.frames.questWatchTooltip;

    tooltip.updateTime = (tooltip.updateTime or 0.2) - elapsed;

    if (tooltip.questID == nil and not tooltip:IsFading() and tooltip:IsVisible()) then
        tooltip:FadeOutTooltip();
    end

    if (tooltip.updateTime > 0 or tooltip.questID == nil) then 
        return;
    end

    tooltip.updateTime = 0.2;

    if (tooltip:IsFading()) then
        tooltip:StopFading();
    end

    local Groupmember = {};

    -- show tooltip
    ShowInspectCursor()
    tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")

    local title, qLvl, tag, isHeader, isCollapsed, isComplete, frequency, questID, index, header = beql:GetQuestLogTitle(tooltip.questID);

    if (beql.db.profile.Tooltip.QuestTracker_TooltipTitleQuestcolor) then
        Color = GetQuestDifficultyColor(qLvl)
    else
        Color = beql.db.profile.Tooltip.Colors.QuestTracker_TooltipTitle
    end
    tooltip:AddLine(title, Color.r, Color.g, Color.b)

    -- Questdetails
    if (beql.db.profile.Tooltip.QuestTracker_TooltipDesc) then
        local questObjectives = select(2, GetQuestLogQuestText(index))
        tooltip:AddLine(questObjectives, beql.db.profile.Tooltip.Colors.QuestTracker_TooltipDesc.r, beql.db.profile.Tooltip.Colors.QuestTracker_TooltipDesc.g, beql.db.profile.Tooltip.Colors.QuestTracker_TooltipDesc.b, 1)
    end

    -- Groupmember
    -- temp fix for display Groupmember on Quest
    if (GetNumGroupMembers() > 0) then
        for i=1, GetNumGroupMembers(), 1 do
            local partymember, partyrealm = UnitName("party"..i)

            if (partyrealm and partyrealm ~= "" and partyrealm ~= GetRealmName()) then
                partymember = partymember .. "-" .. partyrealm
            end

            if (IsUnitOnQuest(index, "party" .. i)) then
                tinsert(Groupmember, partymember);
            end
        end

        if (#Groupmember > 0) then
            tooltip:AddLine(" ", nil, nil, nil, 1)
            tooltip:AddLine(L["PartyMember on Quest:"], 1, 1, 1, 1)
            tooltip:AddLine(table.concat(Groupmember, ', '), nil, nil, nil, 1)
        end
    end

    tooltip:Show()
end

--- EOF ---
